<?php

if (!defined('_ECRIRE_INC_VERSION')) {
	return;
}

include_spip('inc/autoriser');

// ---------------------------------------------------
// fonctions de debuggage
// ---------------------------------------------------

function debug() {
	if (autoriser('dd')) {
		$arg_list = func_get_args();
		foreach ($arg_list as $arg) {
			echo _debug($arg);
		}
	}
}

// debug() + die()
function d() {
	if (autoriser('dd')) {
		$arg_list = func_get_args();
		foreach ($arg_list as $arg) {
			echo _debug($arg);
		}
		die();
	}
}

// à utiliser comme filtre dans un squelette : [(#ENV**|dd)]
function dd($valeur) {
	if (autoriser('dd')) {
		return _debug($valeur);
	}
	return null;
}

function _debug($var, $show_html = true) {
	static $css = false;
	$scope  = false;
	$prefix = 'unique';
	$suffix = 'value';

	if (!$css) {
		$out = '<style type="text/css"> 
		.ddbox{position: relative; text-align: left; margin: 0; display: block; background: #444; color: white; padding: 0.4em; font-size: 0.85em; line-height: 1.5; white-space: pre-wrap; font-family: Consolas, "Andale Mono WT", "Andale Mono", "Lucida Console", "Lucida Sans Typewriter", "DejaVu Sans Mono", "Bitstream Vera Sans Mono", "Liberation Mono", "Nimbus Mono L", Monaco, "Courier New", Courier, monospace; border-radius: 4px; margin-bottom: 1px;}
		</style>';
		$css = true;
	}

	if ($scope) {
		$vals = $scope;
	} else {
		$vals = $GLOBALS;
	}

	$old   = $var;
	$new   = $prefix . rand() . $suffix;
	$vname = false;
	foreach ($vals as $key => $val) {
		if ($val === $new) {
			$vname = $key;
		}
	}
	$var = $old;

	$out .= "<div class='ddbox'>";
	$out .= do_dump($var, $vname, null, null, $show_html);
	$out .= "</div>";

	return $out;
}

function do_dump(&$var, $var_name = null, $indent = null, $reference = null, $show_html = true, $out = '') {
	$out            = '';
	$do_dump_indent = "&nbsp;&nbsp;";
	$reference      = $reference . $var_name;
	$keyvar         = 'the_do_dump_recursion_protection_scheme';
	$keyname        = 'referenced_object_name';

	if (is_array($var) && isset($var[$keyvar])) {
		$real_var  = &$var[$keyvar];
		$real_name = &$var[$keyname];
		$type      = ucfirst(gettype($real_var));
		$out       .= "$indent$var_name <span style='color:#e5e5e5'>$type</span> = <span style='color:#e87800;'>&amp;$real_name</span><br>";
	} else {
		$var  = array(
			$keyvar  => $var,
			$keyname => $reference,
		);
		$avar = &$var[$keyvar];

		$type = ucfirst(gettype($avar));
		if ($type == "String") {
			$type_color = "<span style='color:orange'>";
		} elseif ($type == "Integer") {
			$type_color = "<span style='color:lightcoral'>";
		} elseif ($type == "Double") {
			$type_color = "<span style='color:lightseagreen'>";
			$type       = "Float";
		} elseif ($type == "Boolean") {
			$type_color = "<span style='color:lightpink'>";
		} elseif ($type == "NULL") {
			$type_color = "<span style='color:#ddd'>";
		}

		if (is_array($avar)) {
			$count = count($avar);
			$out   .= "$indent" . ($var_name ? "$var_name => " : "") . "<span style='color:#a2a2a2'>$type ($count)</span><br>$indent(<br>";
			$keys  = array_keys($avar);
			foreach ($keys as $name) {
				$value = &$avar[$name];
				$out   .= do_dump($value, "['$name']", $indent . $do_dump_indent, $reference, $show_html);
			}
			$out .= "$indent)<br>";

		} elseif (is_object($avar)) {
			$out .= "$indent$var_name <span style='color:#a2a2a2'>$type</span><br>$indent(<br>";
			foreach ($avar as $name => $value) {
				$out .= do_dump($value, "$name", $indent . $do_dump_indent, $reference, $show_html);
			}
			$out .= "$indent)<br>";
		} elseif (is_int($avar)) {
			$out .= "$indent$var_name <span style='color:#a2a2a2'>$type(" . strlen($avar) . ")</span> $type_color$avar</span><br>";
		} elseif (is_string($avar)) {
			$out .= "$indent$var_name <span style='color:#a2a2a2'>$type(" . strlen($avar) . ")</span> $type_color\"" . ($show_html ? _dd_show_html($avar) : $avar) . "\"</span><br>";
		} elseif (is_float($avar)) {
			$out .= "$indent$var_name <span style='color:#a2a2a2'>$type(" . strlen($avar) . ")</span> $type_color$avar</span><br>";
		} elseif (is_bool($avar)) {
			$out .= "$indent$var_name <span style='color:#a2a2a2'>$type(" . strlen($avar) . ")</span> $type_color" . ($avar == 1 ? "TRUE" : "FALSE") . "</span><br>";
		} elseif (is_null($avar)) {
			$out .= "$indent$var_name <span style='color:#a2a2a2'>$type(" . strlen($avar) . ")</span> {$type_color}NULL</span><br>";
		} else {
			$out .= "$indent$var_name <span style='color:#a2a2a2'>$type(" . strlen($avar) . ")</span> $avar<br>";
		}
	}
	return $out;
}

function _dd_show_html($html) {
	$html      = htmlspecialchars($html);
	$replace[] = array(
		'&lt;',
		'<span style="color:red;">&lt;',
	);
	$replace[] = array(
		'&gt;',
		'&gt;</span>',
	);
	foreach ($replace as $r) {
		$patterns[]     = '/' . str_replace('/', '\/', $r[0]) . '/';
		$replacements[] = $r[1];
	}
	$html = preg_replace($patterns, $replacements, $html);
	return nl2br($html);
}

function debug_sql() {
	$arg_list    = func_get_args();
	$debug_items = array(
		'SELECT',
		'FROM',
		'WHERE',
		'GROUP BY',
		'ORDER BY',
		'LIMIT',
		'HAVING',
	);
	$debug       = '';
	foreach ($arg_list as $key => $arg) {
		if ($arg) {
			$debug .= $debug_items[$key] . ' ';
			if (is_array($arg)) {
				if ($key === 'WHERE' || $key === 2) {
					$debug .= join(' AND ', $arg);
				} else {
					$debug .= join(', ', $arg);
				}
			} else {
				$debug .= $arg;
			}
			$debug .= ' ' . "\r\n";
		}
	}
	debug(trim($debug));
}

function debug_table($tab = array(), $return = false, $value_legnth = false) {
	$out = '';
	$out .= '<table style="border-collapse: collapse; border: 1px solid gray;">';
	foreach ($tab as $index => $tab2) {

		if ($index == 0) {
			$out .= '<tr>';
			foreach ($tab2 as $key => $val) {
				$out .= '<td style="font-size: 0.8em; border-collapse: collapse; border: 1px solid gray;">' . $key . '</td>';
			}
			$out .= '</tr>';
		}

		$out .= '<tr>';
		foreach ($tab2 as $key => $val) {
			$out .= '<td style="font-size: 0.8em; border-collapse: collapse; border: 1px solid gray;">' . ($val == '' ? '&nbsp;' : ($value_legnth ? couper($val,
					$value_legnth) : $val)) . '</td>';
		}
		$out .= '</tr>';
	}
	$out .= '</table>';
	if (!$return) {
		echo $out;
	} else {
		return $out;
	}
}

//-----------------------------
// Affichage d'infos 
//-----------------------------

function dd_affiche_infos() {
	$headers     = headers_list();
	$header_html = false;
	// tester si un content-type text/html a déjà été passé
	foreach ($headers as $header) {
		$header = strtolower($header);
		if (strpos($header, 'content-type') !== false && strpos($header, 'text/html') !== false) {
			$header_html = true;
		}
	}
	// ne pas afficher en mode cli ou sur un content-type autre que text/html
	if (!$header_html || php_sapi_name() === 'cli') {
		return false;
	}

	if (
		(!isset($GLOBALS['flag_preserver']) || !$GLOBALS['flag_preserver'])
		&& (!defined('_AJAX') || !_AJAX)
		&& (!isset($_SERVER['HTTP_X_REQUESTED_WITH']) || !$_SERVER['HTTP_X_REQUESTED_WITH'])
		&& !_request('var_zajax')
		&& !_request('action')
		&& (!_request('nodd'))
	) {
		chdir(_ROOT_CWD); // precaution

		if (version_compare(PHP_VERSION, '5.4.0') >= 0) {
			$infos[] = 'Page '.number_format((microtime(true) - $_SERVER["REQUEST_TIME_FLOAT"]), 2) . ' s';
		} else {
			if (isset($GLOBALS['page_start_time'])) {
				$infos[] = 'Page '.number_format((microtime(true) - $GLOBALS['page_start_time']), 2) . ' s';
			}
		}

		if (isset($GLOBALS['connexions']) && is_array($GLOBALS['connexions'])) {
			foreach ($GLOBALS['connexions'] as $serveur => $connexion) {
				$info_sql = '';
				if (isset($connexion['requetes']) && $connexion['requetes']) {
					$info_sql .= '<a class="dd_show_queries" data-connexion="' . $serveur . '" href="#">';
				}
				if (isset($connexion['total_requetes'])) {
					$info_sql .= ($serveur ? $serveur . " : " : "") . $connexion['total_requetes'] . " requetes";
				}
				if (isset($connexion['requetes']) && $connexion['requetes']) {
					$info_sql .= '</a>';
					$sql_time = 0;
					$queries  = '<ol class="dd_queries" data-connexion="' . $serveur . '">';
					foreach ($connexion['requetes'] as $query) {
						$sql_time += $query['time'];
						$queries .= "\n" . '<li class="dd_query" data-connexion="' . $serveur . '"><span>' . $query['sql'] . '</span><span class="time">'. number_format($query['time'],6) . '</span></li>';
					}
					$queries  .= '</ol>';
					$info_sql .= $queries;
				}
				$infos[] = $info_sql;
				$infos[] = 'SQL '.number_format($sql_time,2).' s';
			}
		} else {
			$infos[] = "0 requetes :)";
		}

		if (function_exists('memory_get_usage')) {
			$infos[] = round(memory_get_usage() / 1024 / 1024, 1) . ' Mo';
		}

		if (function_exists('sys_getloadavg')) {
			$load    = sys_getloadavg();
			$infos[] = number_format($load[0], 2) . ' / ' . number_format($load[1], 2) . ' / ' . number_format($load[2], 2);
		}

		echo '<div class="debug_info">' . join(' | ', $infos) . '</div>';
		echo <<<EOCSS
			<style>
			.debug_info, .dd_queries {font-size: 90%; background: #343434; color: white;}
			.debug_info a, .dd_queries a {color: white; text-decoration: underline; }
			.debug_info {position: fixed; bottom: 0; right: 0; z-index: 10000; padding: 0.1em 0.8em;}
			.dd_queries {position: fixed; top: 1.5em;right: 1.5em; left: 1.5em; height: calc(100vh - 40px - 1.5em); overflow-y: scroll; display: none; z-index: 10001; list-style: decimal; padding: 1em; word-wrap: break-word; text-align: left; }
			.dd_query { margin-bottom: 1em; display: flex; justify-content: space-between}
			.dd_query .time { margin-left: 1.5em; }
			</style>
EOCSS;

		echo <<<EOJS
			<script>
				$(function(){
					$('.debug_info').on('mousedown', function(e){
						let orignalTarget = e.originalEvent.target; 
						let classlist = orignalTarget.classList;
						if(	classlist.contains('dd_show_queries') || classlist.contains('dd_queries') || classlist.contains('dd_query')){
							// gérer un timing court sur mousedown/up, pour pouvoir sélectionner du texte
							let	down = +new Date();
							let up;
							$(this).on('mouseup', function(e) {
								up = +new Date();
								if((up - down) < 200) {
									let connexion = orignalTarget.dataset.connexion;
									let queries = $('.dd_queries[data-connexion='+connexion+']');
									if(queries.is(':visible')){
										queries.hide();
										$('body').css('overflow','auto');		
									} else {
										queries.show();
										$('body').css('overflow','hidden');								
									}
									e.preventDefault();
								} 
							});
						} else {
							$(this).remove();	
							e.preventDefault();
						}						
					});
				});
			</script>
EOJS;

	}

}
if (autoriser('dd')) {
	register_shutdown_function('dd_affiche_infos');
}